local moduleName = 'Minimap'
local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local module = idTweaks:newModule(moduleName, 'AceConsole-3.0')
local AceGUI = LibStub('AceGUI-3.0')

local TL, TC, TR = 'TOPLEFT', 'TOP', 'TOPRIGHT'
local ML, MC, MR = 'LEFT', 'CENTER', 'RIGHT'
local BL, BC, BR = 'BOTTOMLEFT', 'BOTTOM', 'BOTTOMRIGHT'

module.defaults = {
  profile = {
    enabled = true,

    shape = {
      square = true,
      position = true,
    },

    hide = {
      areaText = true,
      calendarButton = true,
      northTag = true,
      timeDisplay = true,
      worldMapButton = true,
      zoomButtons = true,
    },

    mouse = {
      mouseWheelZooming = true,
      -- rightClickMenu = true,
    },
  }
}

function module:enable()
  module:update()
end

function module:disable()
  module.shape.square:disable()
  module.shape.position:disable()
  module.mouse:disable()
  module.hide:disable()
end

function module:update()
  module.shape.square:check()
  module.shape.position:check()
  module.mouse:check()
  module.hide:check()
end

function module:positioning()
  module.shape.square:check()
  module.shape.position:check()
end

module.hide = {
  frames = {
    -- MinimapTrackingFrame
    -- MinimapMailFrame
    -- MinimapBattlefieldFrame

    areaText = {
      enable = function()
        MinimapBorderTop:Hide()
        MinimapZoneText:Hide()
        MinimapZoneTextButton:Hide()
        MinimapToggleButton:Hide()
      end,
      disable = function()
        MinimapBorderTop:Show()
        MinimapZoneText:Show()
        MinimapZoneTextButton:Show()
        MinimapToggleButton:Show()
      end,
    },

    calendarButton = {
      enable = function()
        GameTimeFrame:Hide()
      end,
      disable = function()
        GameTimeFrame:Show()
      end,
    },

    northTag = {
      enable = function()
        MinimapNorthTag:SetAlpha(0)
      end,
      disable = function()
        MinimapNorthTag:SetAlpha(1)
      end,
    },

    timeDisplay = {
      enable = function()
        TimeManagerClockButton:Hide()
      end,
      disable = function()
        TimeManagerClockButton:Show()
      end,
    },

    worldMapButton = {
      enable = function()
        MiniMapWorldMapButton:Hide()
      end,
      disable = function()
        MiniMapWorldMapButton:Show()
      end,
    },

    zoomButtons = {
      enable = function()
        MinimapZoomIn:Hide()
        MinimapZoomOut:Hide()
      end,

      disable = function()
        MinimapZoomIn:Show()
        MinimapZoomOut:Show()
      end
    }
  }
}

function module.hide:check()
  for name, functions in pairs(module.hide.frames) do
    module.hide:checkFrame(name)
  end
end

function module.hide:disable()
  for name, functions in pairs(module.hide.frames) do
    module.hide.frames[name].disable()
  end
end

function module.hide:checkFrame(name)
  if module.db.profile.hide[name] == true then
    module.hide.frames[name].enable()
  else
    module.hide.frames[name].disable()
  end
end

-- TODO: Rewrite to the style of the rest of the modules.
module.shape = {}

module.shape.square = {}

function module.shape.square:check()
  if module.db.profile.shape.square == true then
    module.shape.square:enable()
  else
    module.shape.square:disable()
  end
end

function module.shape.square.enable()
  Minimap:SetMaskTexture('Interface\\Buttons\\WHITE8x8')
  MinimapBorder:Hide()
end

function module.shape.square.disable()
  Minimap:SetMaskTexture('textures\\MinimapMask')
  MinimapBorder:Show()
end

module.shape.position = {}

function module.shape.position:check()
  local ActionBarsModule = idTweaks:GetModule('ActionBars')

  if ActionBarsModule.db.profile.enabled then
    if module.db.profile.shape.position == true then
      module.shape.position:enable()
    else
      module.shape.position:disable()
    end
  else
    module.shape.position:disable()
  end
end

function module.shape.position.enable()
  Minimap:ClearAllPoints()
  Minimap:SetParent(UIParent)

  local size

  local barMargin     = idTweaks.modules.ActionBars.db.profile.barMargin
  local barPadding    = idTweaks.modules.ActionBars.db.profile.barPadding
  local buttonPadding = idTweaks.modules.ActionBars.db.profile.buttonPadding

  local experienceBarDatabase = idTweaks.modules.ExperienceBar.db.profile

  if experienceBarDatabase.enabled then
    if experienceBarDatabase.fullWidth then
      local _, height = idTweaks.modules.ActionBars:getTotalSize()
      size = height
      Minimap:SetPoint(BL, idTweaksActionBar1, BR, barMargin, barMargin)
    else
      local _, height = idTweaks.modules.ActionBars:getTotalSize()
      size = height + idTweaksExperienceBar:GetHeight()
      Minimap:SetPoint(BL, idTweaksExperienceBar, BR, barMargin, barMargin)
    end
  else
    local _, height = idTweaks.modules.ActionBars:getTotalSize()
    size = height
    Minimap:SetPoint(BL, idTweaksActionBar1, BR, barMargin, barMargin)
  end

  Minimap:SetWidth(size)
  Minimap:SetHeight(size)
end

function module.shape.position.disable()
  Minimap:ClearAllPoints()
  Minimap:SetParent(MinimapCluster)
  Minimap:SetPoint(MC, MinimapCluster, TC, 9, -92)
  Minimap:SetWidth(140)
  Minimap:SetHeight(140)
end

function GetMinimapShape()
  return module.db.profile.shape.square == true and 'SQUARE' or 'ROUND'
end

module.mouse = {}

function module.mouse:check()
  module.mouse:checkMousewheelZoomingEnabled()
  -- module.mouse:checkRightClickMenuEnabled()
end

function module.mouse:disable()
  module.mouse:disableMouseWheelZooming()
  -- module.mouse:disableRightClickMenu()
end

function module.mouse:checkMousewheelZoomingEnabled()
  if module.db.profile.mouse.mouseWheelZooming == true then
    module.mouse:enableMouseWheelZooming()
  else
    module.mouse:disableMouseWheelZooming()
  end
end

function module.mouse:enableMouseWheelZooming()
  Minimap:EnableMouseWheel(true)
  Minimap:SetScript('OnMouseWheel', module.mouse.handleMousewheelZoom)
end

function module.mouse:disableMouseWheelZooming()
  Minimap:EnableMouseWheel(false)
  Minimap:SetScript('OnMouseWheel', nil)
end

function module.mouse.handleMousewheelZoom(frame, delta)
  if delta > 0 and Minimap:GetZoom() < 5 then
    Minimap:SetZoom(Minimap:GetZoom() + 1)
  elseif delta < 0 and Minimap:GetZoom() > 0 then
    Minimap:SetZoom(Minimap:GetZoom() - 1)
  end
end

-- function module.mouse:checkRightClickMenuEnabled()
--   if module.db.profile.mouse.rightClickMenu == true then
--     module.mouse:enableRightClickMenu()
--   else
--     module.mouse:disableRightClickMenu()
--   end
-- end
--
-- function module.mouse:enableRightClickMenu()
--   module.dropdownItems = {
--     A = 'One',
--     B = 'Two',
--     C = 'Three',
--     D = 'Four',
--   }
--
--   local rightClickDropdown = AceGUI:Create('Dropdown')
--   rightClickDropdown:SetCallback('OnClose', function(frame)
--     module:Print('Dropdown:OnClose')
--     AceGUI:Release(frame)
--   end)
--   rightClickDropdown:SetText('AceGUI-3.0 Example')
--   rightClickDropdown:SetLabel('LABEL?')
--   rightClickDropdown:SetList(module.mouse.dropdownItems)
--   rightClickDropdown.frame:SetPoint(MC, Minimap, MC, 0, 0)
--
--   module.rightClickDropdown = rightClickDropdown
--
--   Minimap:SetScript('OnMouseUp', function(frame, button, down)
--     if button == 'RightButton' then
--       module.rightClickDropdown.frame:Show()
--     else
--       Minimap_OnClick(frame, button, down)
--     end
--   end)
-- end
--
-- function module.mouse:disableRightClickMenu()
--   Minimap:SetScript('OnMouseUp', nil)
-- end

module.options = {
  name = moduleName,
  desc = 'Changes how the Minimap works.',
  type = 'group',
  args = {
    enabled = {
      order = 1,
      name = 'Enabled',
      desc = 'Enables / Disables the Module',
      type = 'toggle',
      get = function()
        return module.db.profile.enabled
      end,
      set = function(_, value)
        module.db.profile.enabled = value
        module:update()
        idTweaks:positioning()
      end,
    },

    shape = {
      name = 'Shape',
      desc = 'Shape settings.',
      type = 'group',
      disabled = function()
        return not module.db.profile.enabled
      end,
      args = {
        square = {
          name = 'Square',
          desc = 'Makes the minimap square',
          type = 'toggle',
          get = function()
            return module.db.profile.shape.square
          end,
          set = function(_, value)
            module.db.profile.shape.square = value
            module:update()
            idTweaks:positioning()
          end
        },

        position = {
          name = 'Position',
          desc = 'Move the minimap next to the action bars',
          type = 'toggle',
          disabled = function()
            local ActionBarsModule = idTweaks:GetModule('ActionBars')
            return ActionBarsModule.db.profile.enabled == false
          end,
          get = function()
            return module.db.profile.shape.position
          end,
          set = function(_, value)
            module.db.profile.shape.position = value
            module:update()
            idTweaks:positioning()
          end
        },
      },
    },

    mouse = {
      name = 'Mouse',
      desc = 'Change how the mouse interacts with the Minimap.',
      type = 'group',
      disabled = function()
        return not module.db.profile.enabled
      end,
      args = {
        mouseWheelZooming = {
          name = 'Mouse Wheel',
          desc = 'Enables Zooming the minimap with the Mouse Wheel.',
          type = 'toggle',
          get = function()
            return module.db.profile.mouse.mouseWheelZooming
          end,
          set = function(_, value)
            module.db.profile.mouse.mouseWheelZooming = value
            module:update()
            idTweaks:positioning()
          end
        },

        -- rightClickMenu = {
        --   name = 'Right Click Menu',
        --   desc = 'Enables the right click menu.',
        --   type = 'toggle',
        --   get = function()
        --     return module.db.profile.mouse.rightClickMenu
        --   end,
        --   set = function(_, value)
        --     module.db.profile.mouse.rightClickMenu = value
        --     module.mouse:checkRightClickMenuEnabled()
        --   end
        -- }
      }
    },

    hide = {
      name = 'Hide',
      desc = 'Toggle visibility of parts of the Minimap with these options.',
      type = 'group',
      disabled = function()
        return not module.db.profile.enabled
      end,
      args = {
        areaText = {
          name = 'Area Text',
          type = 'toggle',
          get = function()
            return module.db.profile.hide.areaText
          end,
          set = function(_, value)
            module.db.profile.hide.areaText = value
            module:update()
            idTweaks:positioning()
          end
        },

        calendarButton = {
          name = 'Calendar Button',
          type = 'toggle',
          get = function()
            return module.db.profile.hide.calendarButton
          end,
          set = function(_, value)
            module.db.profile.hide.calendarButton = value
            module:update()
            idTweaks:positioning()
          end
        },

        northTag = {
          name = 'North Tag',
          type = 'toggle',
          get = function()
            return module.db.profile.hide.northTag
          end,
          set = function(_, value)
            module.db.profile.hide.northTag = value
            module:update()
            idTweaks:positioning()
          end
        },

        timeDisplay = {
          name = 'Time Display',
          type = 'toggle',
          get = function()
            return module.db.profile.hide.timeDisplay
          end,
          set = function(_, value)
            module.db.profile.hide.timeDisplay = value
            module:update()
            idTweaks:positioning()
          end
        },

        worldMapButton = {
          name = 'World Map Button',
          type = 'toggle',
          get = function()
            return module.db.profile.hide.worldMapButton
          end,
          set = function(_, value)
            module.db.profile.hide.worldMapButton = value
            module:update()
            idTweaks:positioning()
          end
        },

        zoomButtons = {
          name = 'Zoom Buttons',
          type = 'toggle',
          get = function()
            return module.db.profile.hide.zoomButtons
          end,
          set = function(_, value)
            module.db.profile.hide.zoomButtons = value
            module:update()
            idTweaks:positioning()
          end
        },
      }
    }
  }
}
