local moduleName = 'WarlockDeleteShards'
local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local module = idTweaks:newModule(moduleName, 'AceConsole-3.0', 'AceEvent-3.0')

module.defaults = {
  profile = {
    enabled = true,
    maximumShards = 5,
  }
}

function module:enable()
  module:RegisterEvent('BAG_UPDATE')
end

function module:disable()
  module:UnregisterEvent('BAG_UPDATE')
end

function module:deleteItem(bagnum, slotnum)
  ClearCursor()
  PickupContainerItem(bagnum, slotnum)
  DeleteCursorItem()
  ClearCursor()
end

function module:itemIsShard(x, y)
  local itemLink = GetContainerItemLink(x, y)

  return itemLink and string.find(itemLink, 'Soul Shard')
end

function module:getShardSpace()
  local value = 0

  for i = 0, 4 do
    for j = 1, GetContainerNumSlots(i) do
      if module:itemIsShard(i, j) then
        value = value + 1
      end
    end
  end

  return value
end

function module:BAG_UPDATE()
  local shardSpace = module:getShardSpace()
  local maximumShards = module.db.profile.maximumShards

  for i = 0, 4 do
    for j = 1, GetContainerNumSlots(i) do
      if shardSpace > maximumShards and module:itemIsShard(i, j) then
        shardSpace = shardSpace - 1
        module:deleteItem(i, j)
      end
    end
  end
end

module.options = {
  name = moduleName,
  type = 'group',
  args = {
    enabled = {
      order = 1,
      name = 'Enabled',
      desc = 'Enables / Disables the Module',
      type = 'toggle',
      get = function()
        return module.db.profile.enabled
      end,
      set = function(_, value)
        module.db.profile.enabled = value
        module:check()
      end,
    },

    maximumShards = {
      order = 2,
      name = 'Maximum Shards',
      desc = 'The amount of shards to keep',
      type = 'range',
      min = 1,
      max = 50,
      step = 1,
      bigStep = 1,
      disabled = function()
        return not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.maximumShards
      end,
      set = function(_, value)
        module.db.profile.maximumShards = value
      end,
    },
  }
}
